      program getsphyw
c --- This program generates a cluster of spheres using a larger distribution
c     created by the Visscher-Bolsterli program and written to file "ballout".
c     ndim is a parameter used to dimension the variable in this program.
c     ndist is read from the first line of "ballout". It is the number of 
c     spheres in the large distribution.
c     nsph in the number of spheres desired in the new small distribution.
c     Obviously, nsph <= ndim <= ndist.
c
c**** Modified by Y. M. Wang in July, 1991 to keep the origin of the coordinate
c     system unchanged. Meanwhile, the number of spheres to be taken out has 
c     been determined by the radius of the global ball instead of the number
c     spheres specified in the old version of the program.
c
      parameter (ndim=6500)
      real x(ndim),y(ndim),z(ndim),r(ndim),pho(ndim)
      integer idx(ndim)
      open (7,file='ballout')
      open (8,file='phoorig')
      open (9,file='theorig')
      open (10,file='phiorig')
      open (11,file='radorig')
      open (12,file='sphere#')
      read (7,*) ndist,nshake
      read (7,*) xpar,ypar
      print *,'ndist,xpar,ypar =',ndist,xpar,ypar
      if (ndist.gt.ndim) then 
        print *,'***** ndim not large enough *****'
        stop
      end if
      print *,' '
      print *,'Data file "ballout" contains the positions and radii of',
     &        ndist,' spheres.'
      print *,'Enter radius of the global spheres in your cluster.'
      read *, rg
      print *,' '

c --- zmax is a rather arbitrary number you must enter.
c     To get a rough estimate, look near the end of "ballout" to find the
c     z-coordinates of the spheres near the top of the bin.
c     Pick zmax such that the bin is filled relatively solidly below that
c     value. (Imagine evening the top off by brushing off the spheres which
c     stuck up above the rest and the putting a lid on the bin now that the
c     top is relatively flat. The lid is sitting at a height = zmax.)

      print *,'Enter zmax.'
      read *,zpar
      print *,' '
      print *,'Enter integer ( < 0 ) to initialize random # generator.'
      read *,idum
      print *,' '

c --- Choose a point somewhere near the middle of the distribution.
      xran=ran3(idum)
      yran=ran3(-idum)
      zran=ran3(-2*idum)
      xcen=(xpar-20.)*xran + 10.
      ycen=(ypar-20.)*yran + 10.
      zcen=(zpar-20.)*zran + 10.
     
      orig = 25.0e0

c --- Find the distances of the scatterers to the origin
      do 10 n=1,ndist
        read (7,*) r(n),x(n),y(n),z(n)
        pho(n)=sqrt((x(n)-orig)**2+(y(n)-orig)**2+(z(n)-orig)**2)
  10  continue

c --- Order the distances to central sphere
      call indexx(ndist,pho,idx)
      
      k = 0
      vsum=0.0
      do 2 n=1,ndist
        if(pho(idx(n))+r(idx(n)) .le. rg) then
          k = k + 1
          print *,'pho(',idx(n),') =',pho(idx(n))
          write(11,*) r(idx(n))
          vsum=vsum+(4.*3.1415927/3.0)*(r(idx(n))**3)
          xm = x(idx(n))-orig
          ym = y(idx(n))-orig
          zm = z(idx(n))-orig
          phom   = sqrt(xm**2+ym**2+zm**2)
          thetam = atan2(sqrt(xm**2+ym**2),zm)
          phim   = atan2(ym,xm)
          write(8 ,*) phom
          write(9 ,*) thetam
          write(10,*) phim
          write(12,*)'Sphere',idx(n),' is sphere',n
        endif
  2   continue
      write(12,*)' '
      write(12,*)'Volume occuppied by spheres is',vsum

      sum=0.0
      do 4 i=1,k
           sum=sum+r(i)**3
  4   continue
      fv=sum/rg**3
      print *,' '
      print *,'fractional volume =',fv
      write(12,*)'Fractional Volume =',fv
c      write(12,*)'Group Radius =',((.75/3.1415927)*(vsum/fv))**(1./3.)
      write(12,*)'Group Radius =',rg
      write(11,*) rg
      print *,' '

      stop
      end
c-----------------------------------------------------------------------
      FUNCTION RAN3(IDUM)
C         IMPLICIT REAL*4(M)
C         PARAMETER (MBIG=4000000.,MSEED=1618033.,MZ=0.,FAC=2.5E-7)
      PARAMETER (MBIG=1000000000,MSEED=161803398,MZ=0,FAC=1.E-9)
      DIMENSION MA(55)
      DATA IFF /0/
      IF(IDUM.LT.0.OR.IFF.EQ.0)THEN
        IFF=1
        MJ=MSEED-IABS(IDUM)
        MJ=MOD(MJ,MBIG)
        MA(55)=MJ
        MK=1
        DO 11 I=1,54
          II=MOD(21*I,55)
          MA(II)=MK
          MK=MJ-MK
          IF(MK.LT.MZ)MK=MK+MBIG
          MJ=MA(II)
11      CONTINUE
        DO 13 K=1,4
          DO 12 I=1,55
            MA(I)=MA(I)-MA(1+MOD(I+30,55))
            IF(MA(I).LT.MZ)MA(I)=MA(I)+MBIG
12        CONTINUE
13      CONTINUE
        INEXT=0
        INEXTP=31
        IDUM=1
      ENDIF
      INEXT=INEXT+1
      IF(INEXT.EQ.56)INEXT=1
      INEXTP=INEXTP+1
      IF(INEXTP.EQ.56)INEXTP=1
      MJ=MA(INEXT)-MA(INEXTP)
      IF(MJ.LT.MZ)MJ=MJ+MBIG
      MA(INEXT)=MJ
      RAN3=MJ*FAC
      RETURN
      END

c---------------------------------------------------------------------
      SUBROUTINE INDEXX(N,ARRIN,INDX)
      DIMENSION ARRIN(N),INDX(N)
      DO 11 J=1,N
        INDX(J)=J
11    CONTINUE
      L=N/2+1
      IR=N
10    CONTINUE
        IF(L.GT.1)THEN
          L=L-1
          INDXT=INDX(L)
          Q=ARRIN(INDXT)
        ELSE
          INDXT=INDX(IR)
          Q=ARRIN(INDXT)
          INDX(IR)=INDX(1)
          IR=IR-1
          IF(IR.EQ.1)THEN
            INDX(1)=INDXT
            RETURN
          ENDIF
        ENDIF
        I=L
        J=L+L
20      IF(J.LE.IR)THEN
          IF(J.LT.IR)THEN
            IF(ARRIN(INDX(J)).LT.ARRIN(INDX(J+1)))J=J+1
          ENDIF
          IF(Q.LT.ARRIN(INDX(J)))THEN
            INDX(I)=INDX(J)
            I=J
            J=J+J
          ELSE
            J=IR+1
          ENDIF
        GO TO 20
        ENDIF
        INDX(I)=INDXT
      GO TO 10
      END
