      program rotate
c --- This program rotates a spherical coordinate system.
c     It creates new data files holding the coordinates of old points
c     expressed in the new system.

      parameter (n=4)
      real pho1(n,n),theta1(n,n),phi1(n,n),x1(n,n),y1(n,n),z1(n,n)
      real pho2(n,n),theta2(n,n),phi2(n,n),x2(n,n),y2(n,n),z2(n,n)

c --- Open 3 old data files and 4 new data files.
      open (7,file ='phodat')
      open (8,file ='thedat')
      open (9,file ='phidat')
      open (10,file='pho2dat')
      open (11,file='the2dat')
      open (12,file='phi2dat')
      open (13,file='angles')

      print *,' '
      print *,'*** Number of data points set at',n,' ***'
      print *,' '
    
c --- Read in old spherical coordinates.
      do 1 i=1,n
        do 2 j=1,n
          read (7,*) pho1(i,j)
          read (8,*) theta1(i,j)
          read (9,*) phi1(i,j)
  2     continue
  1   continue
      
c --- Convert old spherical coordinates to old rectangular coordinates.
      do 11 i=1,n
        do 12 j=1,n
          x1(i,j)=pho1(i,j)*sin(theta1(i,j))*cos(phi1(i,j))
          y1(i,j)=pho1(i,j)*sin(theta1(i,j))*sin(phi1(i,j))
          z1(i,j)=pho1(i,j)*cos(theta1(i,j))
 12     continue
 11   continue

      print *,' '
      print *,'Enter Euler angles of rotation: phi,theta,chi.'
      read  *,a1,a2,a3
      print *,' '

c --- Calculate conversion coefficients from Zare's equation (3.36)
c     Reference: Richard N. Zare    Angular Momentum--
c                Understanding Spatial Aspects in Chemistry & Physics
      cxx =  cos(a1)*cos(a2)*cos(a3) - sin(a1)*sin(a3)
      cxy =  sin(a1)*cos(a2)*cos(a3) + cos(a1)*sin(a3)
      cxz = -sin(a2)*cos(a3)
      cyx = -cos(a1)*cos(a2)*sin(a3) - sin(a1)*cos(a3)
      cyy = -sin(a1)*cos(a2)*sin(a3) + cos(a1)*cos(a3)
      cyz =  sin(a2)*sin(a3)
      czx =  cos(a1)*sin(a2)
      czy =  sin(a1)*sin(a2)
      czz =  cos(a2)

c --- Convert old rectangular coordinates to new rectangular coordinates.
      do 21 i=1,n
        do 22 j=1,n
          x2(i,j) = cxx*x1(i,j) + cxy*y1(i,j) + cxz*z1(i,j)
          y2(i,j) = cyx*x1(i,j) + cyy*y1(i,j) + cyz*z1(i,j)
          z2(i,j) = czx*x1(i,j) + czy*y1(i,j) + czz*z1(i,j)
 22     continue
 21   continue
 
c --- Convert new rectangular coordinates to new spherical coordinates.
      do 31 i=1,n
        do 32 j=1,n
          pho2(i,j)   = sqrt( x2(i,j)**2 + y2(i,j)**2 + z2(i,j)**2 )
          theta2(i,j) = atan2( sqrt(x2(i,j)**2 + y2(i,j)**2), z2(i,j) )
          phi2(i,j)   = atan2( y2(i,j), x2(i,j) )
 32     continue
 31   continue

c --- Write new spherical coordinates and rotation angles to output files.
      do 41 i=1,n
        do 42 j=1,n
          write(10,*) pho2(i,j)
          write(11,*) theta2(i,j)
          write(12,*) phi2(i,j)
 42     continue
 41   continue
      write(13,*)'1st rotation: phi   =',a1
      write(13,*)'2nd rotation: theta =',a2
      write(13,*)'3rd rotation: chi   =',a3

      stop
      end
