      program rotate
c --- Rotates spherical coordinate system.
c     For use with data files needed for effective permittivity programs.
c     Rotating coordinate system simulates waves incident upon the cluster
c     of spheres at different angles.
c     Program currently set to perform several random rotations at user's 
c     command.

c --- m=1:  For use with LINEAR algorithm program
c --- m=n:  For use with N-squared algorithm program
c --- n = number of spheres

      parameter (n=18,m=18)
      real pho1(n,m),theta1(n,m),phi1(n,m),x1(n,m),y1(n,m),z1(n,m)
      real pho2(n,m),theta2(n,m),phi2(n,m),x2(n,m),y2(n,m),z2(n,m)

c --- Open 3 old data files and 4 new data files.
      open (7,file ='pho1')
      open (8,file ='the1')
      open (9,file ='phi1')
      open (10,file='pho2')
      open (11,file='the2')
      open (12,file='phi2')
      open (13,file='angles')

      print *,' '
      print *,'       *** Number of data points set at',n,' ***'
      print *,' '
    
c --- Read in old spherical coordinates.
      do 1 i=1,n
        do 2 j=1,m
          read (7,*) pho1(i,j)
          read (8,*) theta1(i,j)
          read (9,*) phi1(i,j)
  2     continue
  1   continue
      
c --- Convert old spherical coordinates to old rectangular coordinates.
      do 11 i=1,n
        do 12 j=1,m
          x1(i,j)=pho1(i,j)*sin(theta1(i,j))*cos(phi1(i,j))
          y1(i,j)=pho1(i,j)*sin(theta1(i,j))*sin(phi1(i,j))
          z1(i,j)=pho1(i,j)*cos(theta1(i,j))
 12     continue
 11   continue

      print *,'Enter number of rotations desired.'
      read *,nrot
      call GETROT(nrot)
      rewind (unit=13)

      do 100 loop=1,nrot

      read (13,*) a1 
      read (13,*) a2
      read (13,*) a3
      print *,'Rotation Angles:',a1,a2,a3

c --- Calculate conversion coefficients from Zare's equation (3.36)
c     Reference: Richard N. Zare    Angular Momentum--
c                Understanding Spatial Aspects in Chemistry & Physics
      cxx =  cos(a1)*cos(a2)*cos(a3) - sin(a1)*sin(a3)
      cxy =  sin(a1)*cos(a2)*cos(a3) + cos(a1)*sin(a3)
      cxz = -sin(a2)*cos(a3)
      cyx = -cos(a1)*cos(a2)*sin(a3) - sin(a1)*cos(a3)
      cyy = -sin(a1)*cos(a2)*sin(a3) + cos(a1)*cos(a3)
      cyz =  sin(a2)*sin(a3)
      czx =  cos(a1)*sin(a2)
      czy =  sin(a1)*sin(a2)
      czz =  cos(a2)

c --- Convert old rectangular coordinates to new rectangular coordinates.
      do 21 i=1,n
        do 22 j=1,m
          x2(i,j) = cxx*x1(i,j) + cxy*y1(i,j) + cxz*z1(i,j)
          y2(i,j) = cyx*x1(i,j) + cyy*y1(i,j) + cyz*z1(i,j)
          z2(i,j) = czx*x1(i,j) + czy*y1(i,j) + czz*z1(i,j)
 22     continue
 21   continue
 
c --- Convert new rectangular coordinates to new spherical coordinates.
      do 31 i=1,n
        do 32 j=1,m
          pho2(i,j)   = sqrt( x2(i,j)**2 + y2(i,j)**2 + z2(i,j)**2 )
          theta2(i,j) = atan2( sqrt(x2(i,j)**2 + y2(i,j)**2), z2(i,j) )
          phi2(i,j)   = atan2( y2(i,j), x2(i,j) )
 32     continue
 31   continue

c --- Write new spherical coordinates to output files.
      do 41 i=1,n
        do 42 j=1,m
          write(10,*) pho2(i,j)
          write(11,*) theta2(i,j)
          write(12,*) phi2(i,j)
 42     continue
 41   continue

 100  continue

      stop
      end
c----------------------------------------------------------------------
      Subroutine GETROT(nrot)
      print *,' '
      print *,'Enter integer ( < 0 ) to initialize random # generator.'
      read *,idum
      print *,' '
      a1ran=ran3(idum)

      do 1 i=1,nrot
        write(13,*)ran3(i)*2.0*3.1415927
        write(13,*)ran3(i)*1.0*3.1415927
        write(13,*)ran3(i)*2.0*3.1415927
  1   continue
        
      print *,nrot,' sets of 3 rotation angles written to "angles"'
      print *,' '
      return
      end
c-----------------------------------------------------------------------
      FUNCTION RAN3(IDUM)
C         IMPLICIT REAL*4(M)
C         PARAMETER (MBIG=4000000.,MSEED=1618033.,MZ=0.,FAC=2.5E-7)
      PARAMETER (MBIG=1000000000,MSEED=161803398,MZ=0,FAC=1.E-9)
      DIMENSION MA(55)
      DATA IFF /0/
      IF(IDUM.LT.0.OR.IFF.EQ.0)THEN
        IFF=1
        MJ=MSEED-IABS(IDUM)
        MJ=MOD(MJ,MBIG)
        MA(55)=MJ
        MK=1
        DO 11 I=1,54
          II=MOD(21*I,55)
          MA(II)=MK
          MK=MJ-MK
          IF(MK.LT.MZ)MK=MK+MBIG
          MJ=MA(II)
11      CONTINUE
        DO 13 K=1,4
          DO 12 I=1,55
            MA(I)=MA(I)-MA(1+MOD(I+30,55))
            IF(MA(I).LT.MZ)MA(I)=MA(I)+MBIG
12        CONTINUE
13      CONTINUE
        INEXT=0
        INEXTP=31
        IDUM=1
      ENDIF
      INEXT=INEXT+1
      IF(INEXT.EQ.56)INEXT=1
      INEXTP=INEXTP+1
      IF(INEXTP.EQ.56)INEXTP=1
      MJ=MA(INEXT)-MA(INEXTP)
      IF(MJ.LT.MZ)MJ=MJ+MBIG
      MA(INEXT)=MJ
      RAN3=MJ*FAC
      RETURN
      END
