      PROGRAM DIPOLE
                   
C    - This program calculates the scattering matrix for N spheres
C      and uses this matrix to solve for the dipole and multipole terms.
C      The program then calculates and outputs the fractional volume,
C      effective dielectric constant, and conductivity for the current
C      grouping. nstot is the total number of spheres the program is
C      currently set to do. The files thedat, phidat, and phodat hold the
C      each spheres position and distance relative to every other
C      sphere for use in calculating the translation coefficients.
C      The file epsdat holds each spheres dielectric constant, the
C      background dielectric, and the frequency. All dielectric constants 
C      are assumed complex. The file raddat holds each spheres radius,
C      and the group radius. The file outdat recieves the output values.
 
      parameter(mdm=15,numt=3,iwa=80,nst=18)
 
c  ***      nst sets total # of spheres in grouping.             ***
c  *** mdm is matrix dimension, numt # terms kept in series.     ***
c  ***               iwa must be >3xmdm                          ***
 
c  **     Parameter mdm must be changed in subroutine cinv        **
c  **   parameter iwa must be changed in cinv if mdm is changed   **
c  **    -- These are the only things that need be changed --     **
c  **    --    to add spheres or vary matrix dimension.    --     **
 
      complex T11(mdm),T21(mdm),Tm1(mdm),Tmmc(mdm,mdm),Tmm(mdm,mdm)
      complex tmc(nst,mdm,mdm),tm(nst,mdm,mdm),am(nst,mdm,mdm)
      complex EPS(nst),EPB,epseff,Q(mdm,mdm),p(mdm,mdm)
      complex Dple,ONE,MONE,TEMP,ai,temp2,W(mdm,mdm),WINV(mdm,mdm)
      real R(nst),PHO(nst,nst),PHI(nst,nst),THETA(nst,nst),wa(iwa),pi

      open(8,file='phidat')
      open(9,file='thedat')
      open(10,file='phodat')
      open(11,file='raddat')
      open(12,file='epsdat')
      open(13,file='outdat')
      open(15,file='dples')

      nstot=nst
      MD=mdm
      NT=numt
      ns=3
      epso=8.8419e-12
      pi=3.1415927
      ai=(0.0,1.0)
      ONE=(1.0,0.0)
      MONE=(-1.0,0.0)
  
c  - Read in sphere position, size, and epsilon data.
c    The last two elements of file epsdat are the background epsilon
c    and the frequency (in Hertz).
 
      do 1 i=1,nstot
        do 2 j=1,nstot
          read(8,*) PHI(i,j)
          read(9,*) THETA(i,j)
          read(10,*) PHO(i,j)
    2   continue
        read(11,*) R(i)
        read(12,3) EPS(i)
    1 continue
      read(11,*) RG
      read(12,3) EPB
      read(12,4) FQ
    3 format(f11.4,f8.5)
    4 format(e9.4)
      fqeo=fq*epso

      write(*,*)  'group radius=',rg
      write(13,*) 'group radius=',rg
      write(*,*)  'frequency=',fq
      write(13,*) 'frequency=',fq
      write(*,*)  'epb=',epb
      write(13,*) 'epb=',epb
      write(*,*)  'eps=',eps(1)
      write(13,*) 'eps=',eps(1)
      write(*,*)
      write(13,*)
      write(*,*)  '      fv         epsr        conductivity'
      write(13,*) '      fv         epsr        conductivity'
 
C    - First calculate T1(1),T2(1),Tm(1). Tm(1) will be used to
C      hold the 1 sphere scattering matrix for the sphere currently
C      being added to the grouping, and will be updated each time
C      a new sphere is added. After this the two sphere scattering 
C      matrices are calculated. Once this has been done the program
C      enters a large loop which calculates the scattering matrices
C      for 3, 4, 5,..., nstot spheres. "If" blocks are used in the
C      loop to enable computation of the various terms in the
C      matrix sums used in the scattering matrix calculations. For
C      example, the first sum in the T33 scattering matrix consists
C      of two terms, while the first sum of T44 consists of three
C      terms. The terms themselves are assigned in the blocks that
C      call subroutine comat for the coefficient matrices, and the
C      T matrices to be used in the next calculation are properly
C      assigned at the end of the loop.
 
      DO 5 K=1,3
      KC=1
      DO 10 I=1,NT
        NQ=(2*I)+1
        DO 20 LL=1,NQ
          TEMP=(1.0/(R(K)**(I+2)))*(EPB*(I+1)+EPS(K)*I)
          IF (K .EQ. 1) then
            T11(KC)=(1.0/TEMP)*(EPS(K)-EPB)*(R(K)**(I-1))*I
          ELSE IF (K .EQ. 2) then
            T21(kC)=(1.0/TEMP)*(EPS(K)-EPB)*(R(K)**(I-1))*I
          ELSE
            Tm1(KC)=(1.0/TEMP)*(EPS(K)-EPB)*(R(K)**(I-1))*I
          END IF
          KC=KC+1
   20   CONTINUE
   10 CONTINUE
    5 CONTINUE   
C
C    - Now calculate (I-W)**(-1), first calculate Anm,Amn coef.
C
      PHI1=PHI(2,1)
      THETA1=THETA(2,1)
      PHO1=PHO(2,1)
      CALL COMAT(mdm,nst,Am,1,MD,PHI1,THETA1,PHO1)
      PHI1=PHI(1,2)
      THETA1=THETA(1,2)
      PHO1=PHO(1,2)
      CALL COMAT(mdm,nst,Am,nstot,MD,PHI1,THETA1,PHO1)
      DO 60 I=1,MD
        DO 70 J=1,MD
        TEMP=(0.0,0.0)
          DO 80 K=1,MD
            TEMP=TEMP+T21(I)*Am(nstot,I,K)*T11(K)*Am(1,K,J)
   80     CONTINUE
          IF (I .EQ. J) then
            W(I,J)=ONE-TEMP
          ELSE 
            W(I,J)=MONE*TEMP
          END IF
   70   CONTINUE
   60 CONTINUE
      CALL CMINVL(W,WINV,WA,MD,MD)
 
C    - Now calculate T22(i,j).
 
      DO 90 I=1,MD
        DO 100 J=1,MD
          TEMP=(0.0,0.0)  
          TEMP2=(0.0,0.0)  
          DO 110 K=1,MD
            IF (K .EQ. J) then
              TEMP=TEMP+WINV(I,K)*T21(K)*(ONE-(Am(nstot,K,J)*T11(J)))
              TEMP2=TEMP2+T21(K)*(ONE-(Am(nstot,K,J)*T11(J)))
            ELSE
              TEMP=TEMP-WINV(I,K)*T21(K)*Am(nstot,K,J)*T11(J)
              TEMP2=TEMP2-T21(K)*Am(nstot,K,J)*T11(J)
            END IF
  110     CONTINUE
          Tmmc(I,J)=TEMP
  100   CONTINUE
   90 CONTINUE
 
c    - And now T12.
 
      DO 120 I=1,MD
        DO 130 J=1,MD
          TEMP=(0.0,0.0)
          DO 140 K=1,MD
            TEMP=TEMP+Am(1,I,K)*Tmmc(K,J)
  140     CONTINUE
          IF (I .EQ. J) then
            Tmc(1,I,J)=T11(I)*(ONE-TEMP)
          ELSE
            Tmc(1,I,J)=T11(I)*(MONE*TEMP)
          END IF
  130   CONTINUE
  120 CONTINUE

      Dple=(Tmc(1,2,2)+Tmmc(2,2))/(rg**3)
      epseff=((1.0+2.0*Dple)/(1.0-Dple))*epb
      fv=((r(1)**3)+(r(2)**3))/(rg**3)
      epsr=real(epseff)
      sigma=(-1.0)*(aimag(epseff))*fqeo*2.0*pi
      write(*,*) fv,epsr,sigma,dple
      write(13,*) fv,epsr,sigma,dple
 
C    - Now run through main loop nstot-2 times to calculate
C      all other scattering matrices and dipole terms. Loop
C      has been written to, as much as possible in fortran,
C      exploit the recursive scheme. To increase the number 
C      spheres up nstot to the new total number, add the
C      appropriate extra Aim, Timc, and Tim matrices, and
C      increase the size of the data matrices. Then add the
C      appropriate lines of code to the "if blocks" below. It
C      should be clear from examination of those blocks what
C      must be added. Note that the Aim matrices need not be
C      stored, but can be recalculated. This will increase
C      computational time, but reduce the amount of memory required.
 
c    - calculate and save Anm ( n=1 to ns-1 , m=ns ) coef.
c      Amn (am(nstot,i,j) coef. will not be needed again so are not
c      saved. Anm coef. are needed again so are assigned and saved
c      on each run through the loop. Note that we only need 
c      the coef. matrices for the number of spheres we are
c      actually calculating, so we only save nstot matrices
c      on the last run through the loop.
 
      mloop=nstot-2
      loop=2
      idx=2

      do 3000 ml=1,mloop
 
c    -first w and Q are zeroed. They will hold the sums used
c     in calculating the scattering matrices, and are recalculated
c     each time a new sphere is added.
c
      do 142 i=1,md
        do 144 j=1,md
          Q(i,j)=(0.0,0.0)
          w(i,j)=(0.0,0.0)
  144   continue
  142 continue
      do 146 i=loop,1,-1
        phi1=phi(ns,i)
        theta1=theta(ns,i)
        pho1=pho(ns,i)
          call comat(mdm,nst,Am,i,md,phi1,theta1,pho1)
  146 continue
      do 1000 ks=loop,1,-1
      PHI1=PHI(ks,ns)
      THETA1=THETA(ks,ns)
      PHO1=PHO(ks,ns)
      CALL COMAT(mdm,nst,Am,nstot,MD,PHI1,THETA1,PHO1)
 
      DO 150 I=1,MD
        DO 160 J=1,MD
          TEMP=(0.0,0.0)
          DO 170 K=1,MD
            if (ks .eq. idx) then
              TEMP=TEMP+Am(nstot,I,K)*Tmmc(K,J)
            else
              TEMP=TEMP+Am(nstot,I,K)*Tmc(ks,K,J)
            end if
  170     CONTINUE
          p(I,J)=TEMP
          Q(I,J)=Q(I,J)+TEMP
  160   CONTINUE
  150 CONTINUE
      DO 180 I=1,MD
        DO 190 J=1,MD
          TEMP=(0.0,0.0)
          DO 200 K=1,MD
            TEMP=TEMP+p(I,K)*Am(ks,K,J)
  200     CONTINUE
        W(I,J)=W(I,J)+TEMP
  190   CONTINUE
  180 CONTINUE

 1000 CONTINUE

      DO 210 I=1,MD
        DO 220 J=1,MD
            IF (I .EQ. J ) then
              W(I,J)=ONE-(W(I,J)*Tm1(I))
            ELSE
              W(I,J)=MONE*(W(I,J)*Tm1(I))
            END IF
  220   CONTINUE
  210 CONTINUE
      CALL CMINVL(W,WINV,WA,MD,MD)
      DO 270 I=1,MD
        DO 280 J=1,MD
          TEMP=(0.0,0.0)
          DO 290 K=1,MD
            IF (K .EQ. J) then
              TEMP=TEMP+WINV(I,K)*Tm1(K)*(ONE-Q(K,J))
            ELSE
              TEMP=TEMP-WINV(I,K)*Tm1(K)*Q(K,J)
            END IF
  290     CONTINUE
          Tmm(I,J)=TEMP
  280   CONTINUE
  270 CONTINUE
 
c    - Now calculate Tim matrices, i=1,m-1. For example if we
c      have just calculated T55 above we now go through this
c      loop 4 times to calculate T15-T45.
 
      do 2000 ks=1,loop
      do 300 i=1,md
        do 310 j=1,md
          temp=(0.0,0.0)
          do 320 k=1,md
            temp=temp+Am(ks,i,k)*Tmm(k,j)
  320     continue
          if (i .eq. j) then
            Q(i,j)=one-temp
          else
            Q(i,j)=mone*temp
          end if
  310   continue
  300 continue
      do 330 i=1,md
        do 340 j=1,md
          temp=(0.0,0.0)
          do 350 k=1,md
            if (ks .eq. idx) then
              temp=temp+Tmmc(i,k)*Q(k,j)
            else
              temp=temp+Tmc(ks,i,k)*Q(k,j)
            end if
  350     continue
            Tm(ks,i,j)=temp
  340   continue
  330 continue
 2000 continue
 
c  - now set new T's equal to current T's so a new set can be calculated.
 
      do 360 i=1,md
        do 370 j=1,md
          Tmmc(i,j)=Tmm(i,j)
          do 372 in=1,ml+1
            Tmc(in,i,j)=Tm(in,i,j)
  372     continue
  370   continue
  360 continue
 
c    - Increment counters, and output results for current sphere grouping.
 
      Dple=Tmm(2,2)
      do 373 in=1,ml+1
        Dple=Dple+Tm(in,2,2)
  373 continue
      Dple=Dple/(rg**3)
      epseff=((1.0+2.0*Dple)/(1.0-Dple))*epb

      sum=0.0
      do 375 in=1,ns
        sum=sum+(r(in)**3)
  375 continue
      fv=sum/(rg**3)
      epsr=real(epseff)
      sigma=(-1.0)*(aimag(epseff))*fqeo*2.0*pi
      write(*,*)
      write(13,*)
      write(*,*) fv,epsr,sigma,dple
      write(13,*) fv,epsr,sigma,dple
      loop=loop+1
      idx=idx+1
      ns=ns+1
 
c    - Now calculate the individual sphere scattering matrix of
c      the next sphere to be added to the grouping, and then go
c      back through the main loop to calculate the new scattering
c      matrices and dipole moments.
 
      kc=1
      do 380 i=1,nt
        nq=(2*i)+1
        do 390 ll=1,nq
          temp=(1.0/(r(ns)**(i+2)))*(epb*(i+1)+eps(ns)*i)
          Tm1(kc)=(1.0/temp)*(eps(ns)-epb)*(r(ns)**(i-1))*i
          kc=kc+1
  390   continue
  380 continue

 3000 continue
      stop
      end
c-----------------------------------------------------------------

      SUBROUTINE COMAT(mdm,nst,a,la,md,phi,theta,pho)
 
c    - This subroutine fills a given coeficient matrix a
c      with the coordinate translation coeficients specified
c      by phi,theta,and pho.
 
      complex saa,anmkl,a
      real phi,theta,pho
      dimension a(nst,mdm,mdm)
      do 10 i=1,md
          do 20 j=1,md
            nn=j+1
            call genidx(nn,n1,n2)
            n=n1
            m=n2
            nn=i+1
            call genidx(nn,n1,n2)
            k=n1
            l=n2
            anmkl=saa(n,m,k,l,phi,theta,pho)
            a(la,i,j)=anmkl
   20     continue
   10   continue
        return
        end

c
c cminvl in matman1 in hftlib may be used instead if one has access to
c IMSLIB.
c
c transfer 'c' to the next line to get determinant on the matrix.
c     subroutine cminvl(ca,cb,cd,wa,n,ia)
      subroutine cminvl(ca,cb,wa,n,ia)
      parameter(mp=15)
      implicit complex (c)
      dimension ca(mp,mp),cb(mp,mp),cdet(2)
c     dimension wa(80)
      dimension wa(150)
c
c     cminvl is a driver program that finds the inverse of a matrix
c     by calling cgefa and cgedi from linpack.
c
c     on entry
c
c         ca     a complex matrix preserved on output.
c
c         cb     a complex matrix containing the inverse
c                on output.
c
c         cd     contains the determinant of the matrix ca on
c                output.  If the determinant is not needed, the
c                lower line can be used for the subroutine statement.
c
c         wa     a work real vector of length at least 3*n.
c
c         n      the order of the matrices  ca  and  cb.
c
c         ia     the actual row dimension of the array declared for ca and cb.
c
c     written by W.C. Chew on May 3, 1987.
c
c     transfer from ca to cb.
c
      do 10 i=1,n
      do 10 j=1,n
      cb(i,j)=ca(i,j)
   10 continue
c
c     calls cgefa to condition the matrix.
c
      call cgefa(cb,ia,n,wa(1),info)
c
c     if info is greater than zero, the matrix is singular.
c
      if(info.gt.0) goto 15
c
c     calls cgedi to find inverse and determinant.
c
      call cgedi(cb,ia,n,wa(1),cdet,wa(n+1),11)
c     cd=cdet(1)*10.0**cdet(2)
      return
   15 continue
c
c     write to terminal on Harris.  Need to be changed on other computers.
c
      write(*,*) ' ** the matrix is singular in cminvl ** '
      return
      end
c
c     the following programs are from linpack.
c
      subroutine cgefa(a,lda,n,ipvt,info)
      integer lda,n,ipvt(1),info
      complex a(lda,1)
c
c     cgefa factors a complex matrix by gaussian elimination.
c
c     cgefa is usually called by cgeco, but it can be called
c     directly with a saving in time if  rcond  is not needed.
c     (time for cgeco) = (1 + 9/n)*(time for cgefa) .
c
c     on entry
c
c        a       complex(lda, n)
c                the matrix to be factored.
c
c        lda     integer
c                the leading dimension of the array  a .
c
c        n       integer
c                the order of the matrix  a .
c
c     on return
c
c        a       an upper triangular matrix and the multipliers
c                which were used to obtain it.
c                the factorization can be written  a = l*u  where
c                l  is a product of permutation and unit lower
c                triangular matrices and  u  is upper triangular.
c
c        ipvt    integer(n)
c                an integer vector of pivot indices.
c
c        info    integer
c                = 0  normal value.
c                = k  if  u(k,k) .eq. 0.0 .  this is not an error
c                     condition for this subroutine, but it does
c                     indicate that cgesl or cgedi will divide by zero
c                     if called.  use  rcond  in cgeco for a reliable
c                     indication of singularity.
c
c     linpack. this version dated 08/14/78 .
c     cleve moler, university of new mexico, argonne national lab.
c
c     subroutines and functions
c
c     blas caxpy,cscal,icamax
c     fortran abs,aimag,real
c
c     internal variables
c
      complex t
      integer icamax,j,k,kp1,l,nm1
c
      complex zdum
      real cabs1
      cabs1(zdum) = abs(real(zdum)) + abs(aimag(zdum))
c
c     gaussian elimination with partial pivoting
c
      info = 0
      nm1 = n - 1
      if (nm1 .lt. 1) go to 70
      do 60 k = 1, nm1
         kp1 = k + 1
c
c        find l = pivot index
c
         l = icamax(n-k+1,a(k,k),1) + k - 1
         ipvt(k) = l
c
c        zero pivot implies this column already triangularized
c
         if (cabs1(a(l,k)) .eq. 0.0e0) go to 40
c
c           interchange if necessary
c
            if (l .eq. k) go to 10
               t = a(l,k)
               a(l,k) = a(k,k)
               a(k,k) = t
   10       continue
c
c           compute multipliers
c
            t = -(1.0e0,0.0e0)/a(k,k)
            call cscal(n-k,t,a(k+1,k),1)
c
c           row elimination with column indexing
c
            do 30 j = kp1, n
               t = a(l,j)
               if (l .eq. k) go to 20
                  a(l,j) = a(k,j)
                  a(k,j) = t
   20          continue
               call caxpy(n-k,t,a(k+1,k),1,a(k+1,j),1)
   30       continue
         go to 50
   40    continue
            info = k
   50    continue
   60 continue
   70 continue
      ipvt(n) = n
      if (cabs1(a(n,n)) .eq. 0.0e0) info = n
      return
      end
c
      subroutine cgedi(a,lda,n,ipvt,det,work,job)
      integer lda,n,ipvt(1),job
      complex a(lda,1),det(2),work(1)
c
c     cgedi computes the determinant and inverse of a matrix
c     using the factors computed by cgeco or cgefa.
c
c     on entry
c
c        a       complex(lda, n)
c                the output from cgeco or cgefa.
c
c        lda     integer
c                the leading dimension of the array  a .
c
c        n       integer
c                the order of the matrix  a .
c
c        ipvt    integer(n)
c                the pivot vector from cgeco or cgefa.
c
c        work    complex(n)
c                work vector.  contents destroyed.
c
c        job     integer
c                = 11   both determinant and inverse.
c                = 01   inverse only.
c                = 10   determinant only.
c
c     on return
c
c        a       inverse of original matrix if requested.
c                otherwise unchanged.
c
c        det     complex(2)
c                determinant of original matrix if requested.
c                otherwise not referenced.
c                determinant = det(1) * 10.0**det(2)
c                with  1.0 .le. cabs1(det(1)) .lt. 10.0
c                or  det(1) .eq. 0.0 .
c
c     error condition
c
c        a division by zero will occur if the input factor contains
c        a zero on the diagonal and the inverse is requested.
c        it will not occur if the subroutines are called correctly
c        and if cgeco has set rcond .gt. 0.0 or cgefa has set
c        info .eq. 0 .
c
c     linpack. this version dated 08/14/78 .
c     cleve moler, university of new mexico, argonne national lab.
c
c     subroutines and functions
c
c     blas caxpy,cscal,cswap
c     fortran abs,aimag,cmplx,mod,real
c
c     internal variables
c
      complex t
      real ten
      integer i,j,k,kb,kp1,l,nm1
c
      complex zdum
      real cabs1
      cabs1(zdum) = abs(real(zdum)) + abs(aimag(zdum))
c
c     compute determinant
c
      if (job/10 .eq. 0) go to 70
         det(1) = (1.0e0,0.0e0)
         det(2) = (0.0e0,0.0e0)
         ten = 10.0e0
         do 50 i = 1, n
            if (ipvt(i) .ne. i) det(1) = -det(1)
            det(1) = a(i,i)*det(1)
c        ...exit
            if (cabs1(det(1)) .eq. 0.0e0) go to 60
   10       if (cabs1(det(1)) .ge. 1.0e0) go to 20
               det(1) = cmplx(ten,0.0e0)*det(1)
               det(2) = det(2) - (1.0e0,0.0e0)
            go to 10
   20       continue
   30       if (cabs1(det(1)) .lt. ten) go to 40
               det(1) = det(1)/cmplx(ten,0.0e0)
               det(2) = det(2) + (1.0e0,0.0e0)
            go to 30
   40       continue
   50    continue
   60    continue
   70 continue
c
c     compute inverse(u)
c
      if (mod(job,10) .eq. 0) go to 150
         do 100 k = 1, n
            a(k,k) = (1.0e0,0.0e0)/a(k,k)
            t = -a(k,k)
            call cscal(k-1,t,a(1,k),1)
            kp1 = k + 1
            if (n .lt. kp1) go to 90
            do 80 j = kp1, n
               t = a(k,j)
               a(k,j) = (0.0e0,0.0e0)
               call caxpy(k,t,a(1,k),1,a(1,j),1)
   80       continue
   90       continue
  100    continue
c
c        form inverse(u)*inverse(l)
c
         nm1 = n - 1
         if (nm1 .lt. 1) go to 140
         do 130 kb = 1, nm1
            k = n - kb
            kp1 = k + 1
            do 110 i = kp1, n
               work(i) = a(i,k)
               a(i,k) = (0.0e0,0.0e0)
  110       continue
            do 120 j = kp1, n
               t = work(j)
               call caxpy(n,t,a(1,j),1,a(1,k),1)
  120       continue
            l = ipvt(k)
            if (l .ne. k) call cswap(n,a(1,k),1,a(1,l),1)
  130    continue
  140    continue
  150 continue
      return
      end
c
      subroutine cgesl(a,lda,n,ipvt,b,job)
      integer lda,n,ipvt(1),job
      complex a(lda,1),b(1)
c
c     cgesl solves the complex system
c     a * x = b  or  ctrans(a) * x = b
c     using the factors computed by cgeco or cgefa.
c
c     on entry
c
c        a       complex(lda, n)
c                the output from cgeco or cgefa.
c
c        lda     integer
c                the leading dimension of the array  a .
c
c        n       integer
c                the order of the matrix  a .
c
c        ipvt    integer(n)
c                the pivot vector from cgeco or cgefa.
c
c        b       complex(n)
c                the right hand side vector.
c
c        job     integer
c                = 0         to solve  a*x = b ,
c                = nonzero   to solve  ctrans(a)*x = b  where
c                            ctrans(a)  is the conjugate transpose.
c
c     on return
c
c        b       the solution vector  x .
c
c     error condition
c
c        a division by zero will occur if the input factor contains a
c        zero on the diagonal.  technically this indicates singularity
c        but it is often caused by improper arguments or improper
c        setting of lda .  it will not occur if the subroutines are
c        called correctly and if cgeco has set rcond .gt. 0.0
c        or cgefa has set info .eq. 0 .
c
c     to compute  inverse(a) * c  where  c  is a matrix
c     with  p  columns
c           call cgeco(a,lda,n,ipvt,rcond,z)
c           if (rcond is too small) go to ...
c           do 10 j = 1, p
c              call cgesl(a,lda,n,ipvt,c(1,j),0)
c        10 continue
c
c     linpack. this version dated 08/14/78 .
c     cleve moler, university of new mexico, argonne national lab.
c
c     subroutines and functions
c
c     blas caxpy,cdotc
c     fortran conjg
c
c     internal variables
c
      complex cdotc,t
      integer k,kb,l,nm1
c
      nm1 = n - 1
      if (job .ne. 0) go to 50
c
c        job = 0 , solve  a * x = b
c        first solve  l*y = b
c
         if (nm1 .lt. 1) go to 30
         do 20 k = 1, nm1
            l = ipvt(k)
            t = b(l)
            if (l .eq. k) go to 10
               b(l) = b(k)
               b(k) = t
   10       continue
            call caxpy(n-k,t,a(k+1,k),1,b(k+1),1)
   20    continue
   30    continue
c
c        now solve  u*x = y
c
         do 40 kb = 1, n
            k = n + 1 - kb
            b(k) = b(k)/a(k,k)
            t = -b(k)
            call caxpy(k-1,t,a(1,k),1,b(1),1)
   40    continue
      go to 100
   50 continue
c
c        job = nonzero, solve  ctrans(a) * x = b
c        first solve  ctrans(u)*y = b
c
         do 60 k = 1, n
            t = cdotc(k-1,a(1,k),1,b(1),1)
            b(k) = (b(k) - t)/conjg(a(k,k))
   60    continue
c
c        now solve ctrans(l)*x = y
c
         if (nm1 .lt. 1) go to 90
         do 80 kb = 1, nm1
            k = n - kb
            b(k) = b(k) + cdotc(n-k,a(k+1,k),1,b(k+1),1)
            l = ipvt(k)
            if (l .eq. k) go to 70
               t = b(l)
               b(l) = b(k)
               b(k) = t
   70       continue
   80    continue
   90    continue
  100 continue
      return
      end
c
      subroutine caxpy(n,ca,cx,incx,cy,incy)
c
c     constant times a vector plus a vector.
c     jack dongarra, linpack, 3/11/78.
c
      complex cx(1),cy(1),ca
      integer i,incx,incy,ix,iy,n
c
      if(n.le.0)return
      if (abs(real(ca)) + abs(aimag(ca)) .eq. 0.0 ) return
      if(incx.eq.1.and.incy.eq.1)go to 20
c
c        code for unequal increments or equal increments
c          not equal to 1
c
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        cy(iy) = cy(iy) + ca*cx(ix)
        ix = ix + incx
        iy = iy + incy
   10 continue
      return
c
c        code for both increments equal to 1
c
   20 do 30 i = 1,n
        cy(i) = cy(i) + ca*cx(i)
   30 continue
      return
      end
      integer function icamax(n,cx,incx)
c
c     finds the index of element having max. absolute value.
c     jack dongarra, linpack, 3/11/78.
c
      complex cx(1)
      real smax
      integer i,incx,ix,n
      complex zdum
      real cabs1
      cabs1(zdum) = abs(real(zdum)) + abs(aimag(zdum))
c
      icamax = 0
      if( n .lt. 1 ) return
      icamax = 1
      if(n.eq.1)return
      if(incx.eq.1)go to 20
c
c        code for increment not equal to 1
c
      ix = 1
      smax = cabs1(cx(1))
      ix = ix + incx
      do 10 i = 2,n
         if(cabs1(cx(ix)).le.smax) go to 5
         icamax = i
         smax = cabs1(cx(ix))
    5    ix = ix + incx
   10 continue
      return
c
c        code for increment equal to 1
c
   20 smax = cabs1(cx(1))
      do 30 i = 2,n
         if(cabs1(cx(i)).le.smax) go to 30
         icamax = i
         smax = cabs1(cx(i))
   30 continue
      return
      end
      subroutine  cscal(n,ca,cx,incx)
c
c     scales a vector by a constant.
c     jack dongarra, linpack,  3/11/78.
c
      complex ca,cx(1)
      integer i,incx,n,nincx
c
      if(n.le.0)return
      if(incx.eq.1)go to 20
c
c        code for increment not equal to 1
c
      nincx = n*incx
      do 10 i = 1,nincx,incx
        cx(i) = ca*cx(i)
   10 continue
      return
c
c        code for increment equal to 1
c
   20 do 30 i = 1,n
        cx(i) = ca*cx(i)
   30 continue
      return
      end
      subroutine  cswap (n,cx,incx,cy,incy)
c
c     interchanges two vectors.
c     jack dongarra, linpack, 3/11/78.
c
      complex cx(1),cy(1),ctemp
      integer i,incx,incy,ix,iy,n
c
      if(n.le.0)return
      if(incx.eq.1.and.incy.eq.1)go to 20
c
c       code for unequal increments or equal increments not equal
c         to 1
c
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        ctemp = cx(ix)
        cx(ix) = cy(iy)
        cy(iy) = ctemp
        ix = ix + incx
        iy = iy + incy
   10 continue
      return
c
c       code for both increments equal to 1
   20 do 30 i = 1,n
        ctemp = cx(i)
        cx(i) = cy(i)
        cy(i) = ctemp
   30 continue
      return
      end
      complex function cdotc(n,cx,incx,cy,incy)
c
c     forms the dot product of two vectors, conjugating the first
c     vector.
c     jack dongarra, linpack,  3/11/78.
c
      complex cx(1),cy(1),ctemp
      integer i,incx,incy,ix,iy,n
c
      ctemp = (0.0,0.0)
      cdotc = (0.0,0.0)
      if(n.le.0)return
      if(incx.eq.1.and.incy.eq.1)go to 20
c
c        code for unequal increments or equal increments
c          not equal to 1
c
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        ctemp = ctemp + conjg(cx(ix))*cy(iy)
        ix = ix + incx
        iy = iy + incy
   10 continue
      cdotc = ctemp
      return
c
c        code for both increments equal to 1
c
   20 do 30 i = 1,n
        ctemp = ctemp + conjg(cx(i))*cy(i)
   30 continue
      cdotc = ctemp
      return
      end
      REAL FUNCTION ALGAMA(XX)

c     from pg. 157 of "Numerical Recipes" by Press,
c     Flannery, Teukolsky, and Vettering.

      real *8 cof(6),stp,half,one,fpf,tmp,ser
      data cof,stp/76.18009173d0,-86.50532033d0,24.01409822d0,
     &   -1.231739516d0,.120858003d-2,-.536382d-5,2.50662827465d0/
      data half,one,fpf/0.5d0,1.0d0,5.5d0/
      algama0=0.0
      pi=3.1415927
      nflag=0
c     xx=-1.5
      if (xx .eq. 0.0) then
        algama=algama0
        go to 1000
      else if (xx .lt. 1.0) then
        xx=(-1.0)*xx+1.0
        nflag=1
      end if
      x=xx-one
      tmp=x+fpf
      tmp=(x+half)*log(tmp)-tmp
      ser=one
      do 10 j=1,6
        x=x+one
        ser=ser+cof(j)/x
 10   continue
      algama=tmp+log(stp*ser)
c     write(*,*) algama
      if (nflag .eq. 1) then
        algama=log(pi)-algama-log(sin(pi*xx))
      end if
c     write(*,*) algama
 1000 return
      end

C....................................................
C
C SAA(N,M,K,L,NS)
C THIS FUNCTION SUBROUTINE CALCULATES THE A(N,M,K,L)
C COEFFICIENT FOR A TRANSLATION OF RX,RY,RZ.
C
      COMPLEX FUNCTION SAA(N,M,K,L,PHI,THETA,PHO)
      COMPLEX SPHERH,SH
      REAL AK,AN,FGM,FAC,PHO,THETA,PHI,CGR2
      AK=FLOAT(K)
      AN=FLOAT(N)
      FGM=EXP(ALGAMA(AN+AK+.5)-ALGAMA(AK+1.5)-ALGAMA(AN+.5))
      FAC=11.136656*(-1)**K*FGM/(PHO**(N+K+1))
      SPHERH=SH(N+K,M-L,cos(theta),cos(phi),sin(phi))
      SAA=FAC*SPHERH*CGR2(N,M,N+K,M-L,K,L)
      RETURN
      END 

C........................................................... 
      REAL FUNCTION DRLGN1(DX,N,M)
C--SUBROUTINE THAT COMPUTES THE ASSOCIATED LEGENDRE FUNCTION.
C  IT TAKES 3 ARGUMENTS: 
C X  WHERE -1<X<1
C N  THE DEGREE
C M  THE ORDER (M<N)
C  THE LEGENDRE FUNCTION IS COMPUTED BASED ON THE SERIES
C  REPRESENTATION AS GIVEN IN 8.812 OF GRADSHTEYN AND RYZHIK,
C  "TABLES OF INTEGRALS, SERIES AND PRODUCTS," 1965, PP.1015.
C
C -----IMPLICIT DOUBLE PRECISION (D)
      IF(IABS(M) .LE. N)GOTO 60
      DRLGN1=0.E0
      RETURN
C
 60   IF((N-M) .EQ. 0)THEN
      DSUM=1.
      GOTO 65
      ENDIF
C
      DSUM=DX**(N-M)
C
 65   DFAC=1.E0
      KMAX=(N-M)
C
      DPRE=ALGAMA(N+.5E0)-ALGAMA(.5E0)-ALGAMA(N-M+1.E0)
C
      IF(M .EQ. 0)THEN
      DPREFAC=1.E0
      GOTO 70
      ENDIF
      DPREFAC=ABS((1.E0-DX*DX))**(M/2.E0)
C
 70   NPREM1=(-1)**M
      DPRE=EXP(DPRE)*NPREM1*DPREFAC
C
C--PRINT *,'LN=',PRE,'1-X2=',PREFAC,'-1M=',PREM1
C
      IF(KMAX .LT. 2)GOTO 90
C
      DO 80K=2,KMAX,2
      DFAC=DFAC*(-1)*(N-M-K+1.)*(N-M-K+2.)/K/(2.*N-K+1.)
      IF((N-M-K) .EQ. 0)THEN
      DF1=1.E0
      GOTO 75
      ENDIF
C
      DF1=DX**(N-M-K)
 75   DSUM=DSUM+DFAC*DF1
 80   CONTINUE
 90   DRLGN1=DSUM*DPRE*2.E0**N
C
      RETURN
      END 

C........................................................
      COMPLEX FUNCTION SH(L,M,DX,DPHICOS,DPHISIN) 
C
C--THIS FUNCTION SUBROUTINE GENERATES SPHERICAL HARMONICS
C OF ORDER L AND M WITH ARGUMENT COS(THE) AND
C (COS(PHI),SIN(PHI)).
C LEGN FUNCTIONS IS USED.
C
C -----IMPLICIT DOUBLE PRECISION (D)
      COMPLEX AI
      REAL PI,DFAC,DRLGN1,DX,DPHICOS,DPHISIN,algama
      INTEGER L,M,MA
C
      AI=(0.0,1.0)
      PI=4.*ATAN(1.)
C
      MA=IABS(M)
C
      DFAC=ALGAMA(L-M+1)-ALGAMA(L+M+1)
      I1=L-MA
      I2=L+MA
      DFAC=1.E0
      IMAX=MAX(I1,I2)
C
      DO 10 I=0,IMAX-1
      IF(I1 .LE. 0)I1=1
      IF(I2 .LE. 0)I2=1
      DFAC=DFAC*I1/I2
      I1=I1-1
      I2=I2-1
 10   CONTINUE
C
C--FAC=ALGAMA(L-M+1)-ALGAMA(L+M+1)
C
      DFAC=DFAC*(2*L+1.E0)/(4*PI)
      DFAC=SQRT(DFAC)
C
C--PRINT *,'X=',X
C
      IF(M .GE. 0)THEN
      SH=DFAC*DRLGN1(DX,L,MA)*((DPHICOS+AI*DPHISIN)**MA)
      sh=sh*(-1)**ma
      ENDIF
C
      IF(M .LT. 0)THEN
      SH=DFAC*DRLGN1(DX,L,MA)*((DPHICOS-AI*DPHISIN)**MA)
      ENDIF
C
      RETURN
      END 

C.......................................................
      FUNCTION CGR2(L,M,L1,M1,L2,M2)
C
C--THIS SUBROUTINE GENERATES THE GAUNT COEFFICIENTS,
C WHICH IS THE INTEGRAL OF THREE SPHERICAL HARMONICS
C WITH THE FIRST ONE CONJUGATED.
C
      PI=3.1415927
C
      CGR2=(-1)**M*SQRT(FLOAT(2*L+1)*
     & FLOAT(2*L1+1)*FLOAT(2*L2+1)/4./PI)*
     & THREEJ(L,L1,L2,0,0,0)*THREEJ(L,L1,L2,-M,M1,M2)
C
      RETURN
      END 

C.................................................
      FUNCTION THREEJ(J1,J2,J3,M1,M2,M3)
C
C--THIS SUBROUTINE GENERATES THE 3-J COEFFICIENTS.
C REF: GOTTFRIED: QUANTUM MECHANICS (VOL. 1)
C
      THREEJ=(-1)**(J1-J2-M3)/SQRT(2.*FLOAT(J3)+1.)*
     & CLGO(J1,M1,J2,M2,J3,-M3)
C
      RETURN
      END 

C..................................................
      FUNCTION CLGO(J1,M1,J2,M2,J,M)
C
C--THIS SUBROUTINE GENERATES THE CG-COEFFICIENTS. 
C REF: M. ABRAMOWITZ, ET.AL., HANDBOOK... ETC.
C
      IF((J1+J2-J) .LT. 0)GOTO 100
      IF((J1-J2+J) .LT. 0)GOTO 100
      IF((-J1+J2+J) .LT. 0)GOTO 100
C
      IF(IABS(M1) .GT. J1)GOTO 100
      IF(IABS(M2) .GT. J2)GOTO 100
      IF(IABS(M) .GT. J)GOTO 100
C
      IF((M1+M2) .NE. M)GOTO 100
C
      SUM=0.
      DO 150 K=0,10000
C
      IF((J1+J2-J-K) .LT. 0)GOTO 200
      IF((J1-M1-K) .LT. 0)GOTO 200
      IF((J2+M2-K) .LT. 0)GOTO 200
C
      IF((J-J2+M1+K) .LT. 0)GOTO 150
      IF((J-J1-M2+K) .LT. 0)GOTO 150
C
      ALFAC=1/2.*(ALGAMA(FLOAT(J1+M1+1))+ALGAMA(FLOAT(J1-M1+1))+
     & ALGAMA(FLOAT(J2+M2+1))+
     & ALGAMA(FLOAT(J2-M2+1))+ALGAMA(FLOAT(J+M+1))+
     & ALGAMA(FLOAT(J-M+1)))-(ALGAMA(FLOAT(K+1))+ 
     & ALGAMA(FLOAT(J1+J2-J-K+1))+
     & ALGAMA(FLOAT(J1-M1-K+1))+ALGAMA(FLOAT(J2+M2-K+1))+
     & ALGAMA(FLOAT(J-J2+M1+K+1))+ALGAMA(FLOAT(J-J1-M2+K+1)))
C
      FAC=EXP(ALFAC)*(-1)**K
      SUM=SUM+FAC
C
 150  CONTINUE
 200  CONTINUE
C
C--PRINT *,'SUM=',SUM
C
      ALFO=1/2.*(ALGAMA(FLOAT(J1+J2-J+1))+ALGAMA(FLOAT(J+J1-J2+1))+
     & ALGAMA(FLOAT(J+J2-J1+1))-ALGAMA(FLOAT(J+J1+J2+2)))
C
      FO=EXP(ALFO)*SQRT(2.*FLOAT(J)+1.) 
      CLGO=SUM*FO
      GOTO 120
C
 100  CLGO=0
 120  RETURN
      END 

c
c    - This subroutine generates the nm indices from a given
c      i matrix index for --sum n from 0 to nt of the sum
c      m from -n to n.
c
      subroutine genidx(nn,n1,n2)
      ic=0 
      do 200 mc=0,nn
        do 210 lc=-mc,mc
          ic=ic+1
          if (ic .eq. nn) go to 220
  210   continue
  200 continue
  220 n1=mc
      n2=lc
      return
      end
