c****module vbassf2
      subroutine vbassf2(r,theta,phi,cvif,ndim,lnp,wk0,jsh)
c
c     In this routine, the basis function for large r is calculted.
c     Since the r-component of the basis function decaying faster
c     than r^2, we have omitted it in the results. This is why
c     the second dimension in cvif if 4. 
c     Meanwhile, since this routine is mainly used for RCS
c     calculation, the explicit dependence on r has been cancelled
c     by multiplying the corresponding terms by r. 
c
      parameter (nudd = 30)
      implicit real (a-b,d-h,o-z)
      implicit complex (c)
c
      dimension cvif(ndim,4)
      dimension csphh(0:nudd,-nudd:nudd)
      dimension pwork(0:nudd)
      dimension chf(0:nudd),sjf(0:nudd)
      dimension cdhf(0:nudd),sdjf(0:nudd)
c
      ci = (0.0e0,1.0e0)
      pi = acos(-1.0e0)
c
      ntp = int(sqrt(float(lnp))) - 1 
c
      acosthe = cos(theta)
      sinth = sin(theta)
      z = r*wk0
c
      call sphharm(acosthe,phi,ntp,ntp,csphh,nudd,nudd,pwork)
c
c****Calculate the bessel functions and their derivatives
c      here chf = h_(l)*r and sjf = j_(l)*r
c
      do 10 jn = 1,ntp
        chf(jn) = exp(ci*(z-0.5*pi*(jn+1)))/wk0
        sjf(jn) = cos(z-0.5*pi*(jn+1))/wk0
        cdhf(jn) = ci*exp(ci*(z-0.5*pi*(jn+1)))
        sdjf(jn) = -sin(z-0.5*pi*(jn+1))
 10   continue

c****The following is for the standing wave basis functions
      if(jsh .eq. 1) then

c
      do 20 jn = 1,ntp
        np = (jn+1)*(jn+1) - jn - 1
        jm = -jn
        cdy = 0.5*sqrt(float((jn-jm)*(jn+jm+1)))
     &       *csphh(jn,jm+1)*exp(-ci*phi)
        csiny = -0.5*cos(theta)*sqrt(float((jn-jm)*(jn+jm+1)))
     &         *csphh(jn,jm+1)*exp(-ci*phi)
     &         +float(jm)*sinth*csphh(jn,jm)
        cvif(np+jm,1) = ci*csiny*sjf(jn)
        cvif(np+jm,2) = -cdy*sjf(jn)
        cvif(np+jm,3) = cdy*sdjf(jn)/wk0
        cvif(np+jm,4) = ci*csiny*sdjf(jn)/wk0
        jm = jn
        cdy = -0.5*sqrt(float((jn+jm)*(jn-jm+1)))
     &       *csphh(jn,jm-1)*exp(ci*phi)
        csiny = 0.5*cos(theta)*(-sqrt(float((jn+jm)*(jn-jm+1)))
     &         *csphh(jn,jm-1)*exp(ci*phi))
     &         +float(jm)*sinth*csphh(jn,jm)
        cvif(np+jm,1) = ci*csiny*sjf(jn)
        cvif(np+jm,2) = -cdy*sjf(jn)
        cvif(np+jm,3) = cdy*sdjf(jn)/wk0
        cvif(np+jm,4) = ci*csiny*sdjf(jn)/wk0
        do 15 jm = -(jn-1),jn-1
          nq = np + jm
          cdy = 0.5*(-sqrt(float((jn+jm)*(jn-jm+1)))
     &         *csphh(jn,jm-1)*exp(ci*phi)
     &         +sqrt(float((jn-jm)*(jn+jm+1)))
     &         *csphh(jn,jm+1)*exp(-ci*phi))
          csiny = 0.5*cos(theta)*(-sqrt(float((jn+jm)*(jn-jm+1)))
     &           *csphh(jn,jm-1)*exp(ci*phi)
     &           -sqrt(float((jn-jm)*(jn+jm+1)))
     &           *csphh(jn,jm+1)*exp(-ci*phi))
     &           +float(jm)*sinth*csphh(jn,jm)
          cvif(np+jm,1) = ci*csiny*sjf(jn)
          cvif(np+jm,2) = -cdy*sjf(jn)
          cvif(np+jm,3) = cdy*sdjf(jn)/wk0
          cvif(np+jm,4) = ci*csiny*sdjf(jn)/wk0
  15    continue
  20  continue
c
c****The following is for the outgoing wave basis functions
      else if(jsh .eq. 2) then
c
      do 120 jn = 1,ntp
        np = (jn+1)*(jn+1) - jn - 1
        jm = -jn
        cdy = 0.5*sqrt(float((jn-jm)*(jn+jm+1)))
     &       *csphh(jn,jm+1)*exp(-ci*phi)
        csiny = -0.5*cos(theta)*sqrt(float((jn-jm)*(jn+jm+1)))
     &         *csphh(jn,jm+1)*exp(-ci*phi)
     &         +float(jm)*sinth*csphh(jn,jm)
        cvif(np+jm,1) = ci*csiny*chf(jn)
        cvif(np+jm,2) = -cdy*chf(jn)
        cvif(np+jm,3) = cdy*cdhf(jn)/wk0
        cvif(np+jm,4) = ci*csiny*cdhf(jn)/wk0
        jm = jn
        cdy = -0.5*sqrt(float((jn+jm)*(jn-jm+1)))
     &       *csphh(jn,jm-1)*exp(ci*phi)
        csiny = 0.5*cos(theta)*(-sqrt(float((jn+jm)*(jn-jm+1)))
     &         *csphh(jn,jm-1)*exp(ci*phi))
     &         +float(jm)*sinth*csphh(jn,jm)
        cvif(np+jm,1) = ci*csiny*chf(jn)
        cvif(np+jm,2) = -cdy*chf(jn)
        cvif(np+jm,3) = cdy*cdhf(jn)/wk0
        cvif(np+jm,4) = ci*csiny*cdhf(jn)/wk0
        do 115 jm = -(jn-1),jn-1
          nq = np + jm
          cdy = 0.5*(-sqrt(float((jn+jm)*(jn-jm+1)))
     &         *csphh(jn,jm-1)*exp(ci*phi)
     &         +sqrt(float((jn-jm)*(jn+jm+1)))
     &         *csphh(jn,jm+1)*exp(-ci*phi))
          csiny = 0.5*cos(theta)*(-sqrt(float((jn+jm)*(jn-jm+1)))
     &           *csphh(jn,jm-1)*exp(ci*phi)
     &           -sqrt(float((jn-jm)*(jn+jm+1)))
     &           *csphh(jn,jm+1)*exp(-ci*phi))
     &           +float(jm)*sinth*csphh(jn,jm)
          cvif(np+jm,1) = ci*csiny*chf(jn)
          cvif(np+jm,2) = -cdy*chf(jn)
          cvif(np+jm,3) = cdy*cdhf(jn)/wk0
          cvif(np+jm,4) = ci*csiny*cdhf(jn)/wk0
 115    continue
 120  continue
c
      endif
c
      return
      end
